/* TroiaPreview
 * version 0.2 date 2004-09-01
 * Copyright (C) 2003-2004 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Font.h"
#include "Globals.h"
#include "Pixel.h"
#include "Frame.h"

//current frame
unsigned char Wall1[PixelZ][PixelX];
unsigned char Wall2[PixelZ][PixelY];
unsigned char Wall3[PixelZ][PixelX];
unsigned char Wall4[PixelZ][PixelY];
unsigned char Ceiling[PixelY][PixelX];
unsigned char Floor[PixelY][PixelX];

//initialize frame with constant color
void FrameSet( unsigned char Color )
{
	int X, Y, Z;

	for( Z = 0; Z < PixelZ; Z++ )
	{
		for( X = 0; X < PixelX; X++ )
		{
			Wall1[Z][X] = Color;
			Wall3[Z][X] = Color;
		}
		for( Y = 0; Y < PixelY; Y++ )
		{
			Wall2[Z][Y] = Color;
			Wall4[Z][Y] = Color;
		}
	}
	for( Y = 0; Y < PixelY; Y++ )
	{
		for( X = 0; X < PixelX; X++ )
		{
			Ceiling[Y][X] = Color;
			Floor[Y][X] = Color;
		}
	}
}

//initialize frame with random data
void FrameRandomize( unsigned char ColorMin, unsigned char ColorMax )
{
	unsigned char Color;
	int Range, X, Y, Z;

	//correct minimum and maximum color
	if( ColorMin > ColorMax )
	{
		Color = ColorMin;
		ColorMin = ColorMax;
		ColorMax = Color;
	}
	//calculate color range
	Range = ColorMax - ColorMin;

	for( Z = 0; Z < PixelZ; Z++ )
	{
		for( X = 0; X < PixelX; X++ )
		{
			Wall1[Z][X] = (unsigned char)(rand( ) % Range) + ColorMin;
			Wall3[Z][X] = (unsigned char)(rand( ) % Range) + ColorMin;
		}
		for( Y = 0; Y < PixelY; Y++ )
		{
			Wall2[Z][Y] = (unsigned char)(rand( ) % Range) + ColorMin;
			Wall4[Z][Y] = (unsigned char)(rand( ) % Range) + ColorMin;
		}
	}
	for( Y = 0; Y < PixelY; Y++ )
	{
		for( X = 0; X < PixelX; X++ )
		{
			Ceiling[Y][X] = (unsigned char)(rand( ) % Range) + ColorMin;
			Floor[Y][X] = (unsigned char)(rand( ) % Range) + ColorMin;
		}
	}
}

//set a pixel
void FrameSetPixel( tPixel3D Pix, unsigned char Color )
{
	switch( Pix.Wall )
	{
		case 1:
			if( Pix.X < PixelX && Pix.Y < PixelZ )
				Wall1[Pix.Y][Pix.X] = Color;
			break;
		case 2:
			if( Pix.X < PixelY && Pix.Y < PixelZ )
				Wall2[Pix.Y][Pix.X] = Color;
			break;
		case 3:
			if( Pix.X < PixelX && Pix.Y < PixelZ )
				Wall3[Pix.Y][Pix.X] = Color;
			break;
		case 4:
			if( Pix.X < PixelY && Pix.Y < PixelZ )
				Wall4[Pix.Y][Pix.X] = Color;
			break;
		case 5:
			if( Pix.X < PixelX && Pix.Y < PixelY )
				Ceiling[Pix.Y][Pix.X] = Color;
			break;
		case 6:
			if( Pix.X < PixelX && Pix.Y < PixelY )
				Floor[Pix.Y][Pix.X] = Color;
			break;
	}
}

//output text to frame
//top left corner of text is at pixel TopLeft
//write in direction Dir (encoded as the number on numpad (e.g. 8 for up))
//if a color is not valid (0..255), it will not be drawn
void FrameTextOut( char * pTxt, tPixel3D TopLeft, unsigned char Dir, unsigned short ForeColor, unsigned short BackColor )
{
	tPixel3D Pix;
	unsigned char Down;
	unsigned char CharFontData[FontSizeY];
	int X, Y;
	unsigned short Color;

	//output all characters
	for( ; *pTxt != 0; pTxt++ )
	{
		//get pointer to font data for this character
		for( Y = 0; Y < FontSizeY; Y++ )
			CharFontData[Y] = FontData[*pTxt][Y];
		//output all columns of character
		for( X = FontSizeX - 1; X >= 0; X-- )
		{
			//start downwards at top of row
			Pix = TopLeft;
			Down = DirRotate( Dir, 2, &Pix );
			//output all rows of character
			for( Y = 0; Y < FontSizeY; Y++ )
			{
				//set pixel
				Color = CharFontData[Y] & (1 << X) ? ForeColor : BackColor;
				if( Color < 256 )
					FrameSetPixel( Pix, (unsigned char)Color );
				//go one pixel down
				Pixel3DMove( Pix, Down, &Pix, &Down );
			}
			//go one pixel right
			Pixel3DMove( TopLeft, Dir, &TopLeft, &Dir );
		}
	} //for( ; *pTxt ...
}
